from barkshark_http_async import View
from barkshark_http_async.server import error as Error


class Service(View):
	__path__ = [
		'/api/service/{name}',
		'/api/service/{name}/{action}'
	]


	async def get(self, request, response, name=None, action=None):
		func = None
		error = None

		if not name:
			return response.set_json({'error': 'Missing service name or action'})

		try:
			proc = self.app['procs'][name]

		except KeyError:
			return response.set_json({'error': f'Not a service: {name}'})

		if not action:
			action = 'info'

		try:
			func = self.get_method(request.method, action)
		except Exception as e:
			error = f'{e.__class__.__name__}: {e}'

		if 'json' in request.headers.get('accept', '').one().lower():
			if error:
				return response.set_json({'error': error})

			if not func:
				return response.set_json({'error': f'Invalid action: {action}'})

			return response.set_json(func(proc) or {'result': 'ok'})

		if error:
			raise Error.InternalServerError(error)

		if not func:
			raise Error.NotFound(f'Invalid action: {action}')

		func(proc)
		raise Error.Found('/')


	async def post(self, request, response, name=None, action=None):
		return await self.get(request, response, name, action)


	def get_method(self, method, action):
		return getattr(self, f'action_{method.lower()}_{action}', None)


	def action_get_info(self, proc):
		return proc.config


	def action_get_start(self, proc):
		proc.start()


	def action_get_stop(self, proc):
		proc.stop()


	def action_get_restart(self, proc):
		proc.restart()


	def action_get_enable(self, proc):
		proc.update_config('enabled', True)


	def action_get_disable(self, proc):
		proc.update_config('enabled', False)
