import asyncio, platform

from barkshark_http_async.server import Application
from izzylib import logging

from . import web, api, middleware
from .config import config, path, password
from .functions import Processes, handle_shutdown


app = Application(
	name = 'PyPanel',
	title = f'PyPanel: {platform.node()}',
	version = '0.0.1',
	git_repo = 'https://git.barkshark.xyz/izaliamae/pypanel',
	listen = config.host,
	port = config.port,
	proto = 'http',
	access_log = logging.get_config('level') <= logging.Levels.VERBOSE,
	tpl_search = [path.script.join('frontend')],
	sig_handler = handle_shutdown
)

app.add_view(web.Home)
app.add_view(web.Edit)
app.add_view(web.Style)
app.add_view(api.Service)

if config.pass_protect:
	app.add_view(web.Login)
	app.add_middleware(middleware.Auth)


def main():
	proc = Processes(app)

	if config.pass_protect:
		logging.info(f'Login to the panel via http://{"localhost" if config.host == "0.0.0.0" else config.host}:{config.port}/login?password={password}')

	app.start()
