import shlex

from barkshark_http_async import View
from barkshark_http_async.server import error
from datetime import timedelta
from izzylib import boolean

from .config import password


class Home(View):
	__path__ = '/'

	async def get(self, request, response):
		return response.set_template('page/home.haml', context={'procs': self.app['procs'].values()})


class Edit(View):
	__path__ = '/edit/{name}'

	async def get(self, request, response, name=None):
		if not name:
			raise error.NotFound('Service not specified')

		try:
			service = dict(self.app['procs'][name].config)
		except KeyError:
			raise error.NotFound(f'Service not found: {name}')

		service['arguments'] = " ".join(service['arguments'])
		service['executable'] = str(service['executable'])
		service['workdir'] = str(service['workdir'])

		return response.set_template('page/edit.haml', context={'service': service})


	async def post(self, request, response, name=None):
		if not name:
			raise error.NotFound('Service not specified')

		try:
			service = self.app['procs'][name]
		except KeyError:
			raise error.NotFound(f'Service not found: {name}')

		for key in service.config:
			value = request.data.form.get(key)

			if not value and key not in ['enabled', 'autorestart']:
				continue

			service[key] = value

		service.config.save()
		raise error.Found('/')


class Login(View):
	__path__ = '/login'

	async def get(self, request, response, message=None, error=None):
		cookie_pass = request.cookies.get('password')

		if cookie_pass and password == cookie_pass:
			raise error.Found('/')

		req_pass = request.data.query.get('password')

		if req_pass:
			return await self.post(request, response, req_pass)

		context = dict(
			message = message,
			error = error
		)

		return response.set_template('page/login.haml', context=context)


	async def post(self, request, response, req_pass=None):
		redir = request.data.form.get('redir', '/')

		if not req_pass:
			req_pass = request.data.form.get('password')

		if not req_pass:
			return await self.get(request, response, error='Empty password')

		elif password != req_pass:
			return await self.get(request, response, error='Invalid password')

		response.set_cookie('password', password,
			maxage = timedelta(weeks=2),
			samesite = True
		)

		raise error.Found(redir)


class Style(View):
	__path__ = '/style.css'

	async def get(self, request, response):
		return response.set_template('style.css')


class Manifest(View):
	__path__ = '/framework/manifest.json'

	async def get(self, request, response):
		data = {
			'name': self.cfg.name,
			'short_name': self.cfg.name.replace(' ', ''),
			'description': 'UvU',
			'icons': [
				{
					'src': "/framework/static/icon512.png",
					'sizes': '512x512',
					'type': 'image/png'
				},
				{
					'src': "/framework/static/icon64.png",
					'sizes': '64x64',
					'type': 'image/png'
				}
			],
			'theme_color': str(response.default_theme.primary),
			'background_color': str(response.default_theme.background),
			'display': 'standalone',
			'start_url': '/',
			'scope': f'{self.cfg.proto}://{self.cfg.web_host}'
		}

		return response.set_json(data)
